/**
 * MediaWiki Api for Axios
 * Provides the API call methods similar to `mw.Api` at non-mw environments
 *
 * @author Dragon-Fish <dragon-fish@qq.com>
 * @license MIT
 */
import { Ref } from '@vue/reactivity';
import { LylaAdapterMeta, LylaRequestOptions, LylaResponse } from './modules/lyla-adapter-fetch';
import { Lyla } from '@lylajs/core';
type LylaResponseWith<T> = LylaResponse<T> & {
    data: T;
};
export declare class MediaWikiApi {
    #private;
    baseURL: Ref<string>;
    cookies: Record<string, string>;
    constructor(baseURL?: string, options?: LylaRequestOptions);
    static normalizeParamValue(item: MwApiParams[keyof MwApiParams]): string | File | undefined;
    static createLylaInstance(baseURL: string, options?: LylaRequestOptions): Lyla<undefined, LylaAdapterMeta>;
    /** Syntactic Sugar */
    get request(): Lyla<any, LylaAdapterMeta>;
    get defaultOptions(): LylaRequestOptions;
    set defaultOptions(options: LylaRequestOptions);
    get defaultParams(): MwApiParams;
    set defaultParams(params: MwApiParams);
    /** Base methods encapsulation */
    get<T = any>(query: MwApiParams, options?: LylaRequestOptions): Promise<LylaResponseWith<T>>;
    post<T = any>(data: MwApiParams, options?: LylaRequestOptions): Promise<LylaResponseWith<T>>;
    login(lgname: string, lgpassword: string, params?: MwApiParams): Promise<{
        result: 'Success' | 'NeedToken' | 'WrongToken' | 'Failed';
        token?: string;
        reason?: {
            code: string;
            text: string;
        };
        lguserid: number;
        lgusername: string;
    }>;
    getUserInfo(): Promise<{
        id: number;
        name: string;
        groups: string[];
        rights: string[];
        blockid?: number | undefined;
        blockedby?: string | undefined;
        blockedbyid?: number | undefined;
        blockedtimestamp?: string | undefined;
        blockreason?: string | undefined;
        blockexpiry?: string | undefined;
    }>;
    /** Token Handler */
    getTokens(type?: MwTokenName[]): Promise<Record<string, string>>;
    token(type?: MwTokenName, noCache?: boolean): Promise<string>;
    postWithToken<T = any>(tokenType: MwTokenName, body: MwApiParams, options?: {
        tokenName?: string;
        retry?: number;
        noCache?: boolean;
    }): Promise<LylaResponseWith<T>>;
    postWithEditToken<T = any>(body: MwApiParams): Promise<LylaResponseWith<T>>;
    getMessages(ammessages: string[], amlang: string | undefined, options: MwApiParams): Promise<Record<string, string>>;
    parseWikitext(wikitext: string, title?: string, extraBody?: MwApiParams, options?: LylaRequestOptions): Promise<string>;
}
export declare class MediaWikiForeignApi extends MediaWikiApi {
    constructor(baseURL?: string, options?: LylaRequestOptions);
}
export default MediaWikiApi;
export { MediaWikiApi as MwApi, MediaWikiForeignApi as ForeignApi };
export type MwApiParams = Record<string, string | number | string[] | undefined | boolean | File>;
export type MwTokenName = 'createaccount' | 'csrf' | 'login' | 'patrol' | 'rollback' | 'userrights' | 'watch';
