"use strict";
/**
 * MediaWiki Api for Axios
 * Provides the API call methods similar to `mw.Api` at non-mw environments
 *
 * @author Dragon-Fish <dragon-fish@qq.com>
 * @license MIT
 */
var _MediaWikiApi_requestHandler, _MediaWikiApi_defaultOptions, _MediaWikiApi_defaultParams, _MediaWikiApi_tokens;
Object.defineProperty(exports, "__esModule", { value: true });
exports.ForeignApi = exports.MwApi = exports.MediaWikiForeignApi = exports.MediaWikiApi = void 0;
const tslib_1 = require("tslib");
const reactivity_1 = require("@vue/reactivity");
const lyla_adapter_fetch_1 = require("./modules/lyla-adapter-fetch");
class MediaWikiApi {
    constructor(baseURL, options) {
        var _a, _b;
        _MediaWikiApi_requestHandler.set(this, void 0);
        _MediaWikiApi_defaultOptions.set(this, void 0);
        _MediaWikiApi_defaultParams.set(this, void 0);
        _MediaWikiApi_tokens.set(this, void 0);
        this.cookies = {};
        // For MediaWiki environment
        if (!baseURL && typeof window === 'object' && window.mediaWiki) {
            const scriptPath = (_b = (_a = window.mediaWiki) === null || _a === void 0 ? void 0 : _a.config) === null || _b === void 0 ? void 0 : _b.get('wgScriptPath');
            typeof scriptPath === 'string' && (baseURL = `${scriptPath}/api.php`);
        }
        if (typeof baseURL !== 'string') {
            throw new Error('baseURL is undefined');
        }
        // Init
        this.baseURL = (0, reactivity_1.ref)(baseURL);
        tslib_1.__classPrivateFieldSet(this, _MediaWikiApi_tokens, {}, "f");
        tslib_1.__classPrivateFieldSet(this, _MediaWikiApi_defaultParams, (0, reactivity_1.ref)({}), "f");
        tslib_1.__classPrivateFieldSet(this, _MediaWikiApi_defaultOptions, (0, reactivity_1.ref)({}), "f");
        // Set default values
        this.defaultParams = {
            action: 'query',
            errorformat: 'plaintext',
            format: 'json',
            formatversion: 2,
        };
        this.defaultOptions = options || {};
        tslib_1.__classPrivateFieldSet(this, _MediaWikiApi_requestHandler, (0, reactivity_1.computed)(() => {
            var _a, _b, _c, _d, _e;
            var _f, _g, _h;
            const options = {
                ...tslib_1.__classPrivateFieldGet(this, _MediaWikiApi_defaultOptions, "f").value,
            };
            (_a = options.hooks) !== null && _a !== void 0 ? _a : (options.hooks = {});
            (_b = (_f = options.hooks).onInit) !== null && _b !== void 0 ? _b : (_f.onInit = []);
            (_c = (_g = options.hooks).onBeforeRequest) !== null && _c !== void 0 ? _c : (_g.onBeforeRequest = []);
            (_d = (_h = options.hooks).onAfterResponse) !== null && _d !== void 0 ? _d : (_h.onAfterResponse = []);
            // Inject default query params
            (_e = options.hooks.onInit) === null || _e === void 0 ? void 0 : _e.unshift((ctx) => {
                var _a;
                // @ts-ignore FIXME: Type error during vite build, too bad!
                ctx.query = {
                    ...tslib_1.__classPrivateFieldGet(this, _MediaWikiApi_defaultParams, "f").value,
                    ...ctx.query,
                };
                // Fix baseURL
                !ctx.url && (ctx.url = this.baseURL.value);
                try {
                    ctx.url = new URL(ctx.url, this.baseURL.value.startsWith('http')
                        ? this.baseURL.value
                        : (_a = globalThis.location) === null || _a === void 0 ? void 0 : _a.href).toString();
                }
                catch (_) { }
                return ctx;
            });
            // Handle cookies for Node.js
            if (!('document' in globalThis)) {
                options.hooks.onBeforeRequest.push((ctx) => {
                    ctx.headers = ctx.headers || {};
                    ctx.headers['cookie'] = Object.keys(this.cookies)
                        .map((name) => `${name}=${this.cookies[name]}`)
                        .join(';');
                    return ctx;
                });
                options.hooks.onAfterResponse.push((ctx) => {
                    const cookieHeaders = ctx.detail.headers.get('set-cookie');
                    const rawCookies = cookieHeaders === null || cookieHeaders === void 0 ? void 0 : cookieHeaders.split(',').map((i) => i.trim());
                    rawCookies === null || rawCookies === void 0 ? void 0 : rawCookies.forEach((i) => {
                        const [name, ...value] = i.split(';')[0].split('=');
                        this.cookies[name] = value.join('=');
                    });
                    return ctx;
                });
            }
            return MediaWikiApi.createLylaInstance(this.baseURL.value, options);
        }), "f");
    }
    static normalizeParamValue(item) {
        if (Array.isArray(item)) {
            return item.join('|');
        }
        else if (typeof item === 'boolean') {
            return item ? '1' : undefined;
        }
        else if (typeof item === 'number') {
            return '' + item;
        }
        else {
            return item;
        }
    }
    static createLylaInstance(baseURL, options = {}) {
        var _a, _b, _c, _d, _e;
        var _f, _g, _h, _j;
        (_a = options.hooks) !== null && _a !== void 0 ? _a : (options.hooks = {});
        (_b = (_f = options.hooks).onInit) !== null && _b !== void 0 ? _b : (_f.onInit = []);
        (_c = (_g = options.hooks).onBeforeRequest) !== null && _c !== void 0 ? _c : (_g.onBeforeRequest = []);
        (_d = (_h = options.hooks).onAfterResponse) !== null && _d !== void 0 ? _d : (_h.onAfterResponse = []);
        (_e = (_j = options.hooks).onResponseError) !== null && _e !== void 0 ? _e : (_j.onResponseError = []);
        options.hooks.onInit.push((ctx) => {
            // console.info(
            //   '[onInit] beforeTransform',
            //   ctx.method,
            //   ctx.url,
            //   ctx.query,
            //   ctx.body
            // )
            var _a, _b, _c, _d;
            var _e, _f;
            if (((_a = ctx.method) === null || _a === void 0 ? void 0 : _a.toLowerCase()) !== 'post') {
                return ctx;
            }
            // Transform json to formdata
            if (ctx.json) {
                const form = new URLSearchParams('');
                for (const key in ctx.json) {
                    const data = MediaWikiApi.normalizeParamValue(ctx.json[key]);
                    if (typeof data === 'undefined')
                        continue;
                    form.append(key, '' + data);
                }
                ctx.body = form;
                ctx.json = undefined;
            }
            if ((globalThis.FormData && ctx.body instanceof FormData) ||
                ctx.body instanceof URLSearchParams) {
                const body = ctx.body;
                // Adjust params
                body.forEach((value, key) => {
                    const data = MediaWikiApi.normalizeParamValue(value);
                    if (typeof data === 'undefined' || data === null) {
                        body.delete(key);
                    }
                    else if (data !== value) {
                        body.set(key, data);
                    }
                });
                // Adjust query
                (_b = ctx.query) !== null && _b !== void 0 ? _b : (ctx.query = {});
                (_c = (_e = ctx.query).format) !== null && _c !== void 0 ? _c : (_e.format = '' + body.get('format') || 'json');
                (_d = (_f = ctx.query).formatversion) !== null && _d !== void 0 ? _d : (_f.formatversion = '' + body.get('formatversion') || '2');
                body.has('origin') && (ctx.query.origin = '' + body.get('origin'));
            }
            return ctx;
        });
        // Adjust query
        options.hooks.onInit.push((ctx) => {
            var _a;
            (_a = ctx.query) !== null && _a !== void 0 ? _a : (ctx.query = {});
            for (const key in ctx.query) {
                const data = MediaWikiApi.normalizeParamValue(ctx.query[key]);
                if (typeof data === 'undefined' || data === null) {
                    delete ctx.query[key];
                }
                else if (data !== ctx.query[key]) {
                    ctx.query[key] = '' + data;
                }
            }
            // console.info('[onInit]', ctx.method?.toUpperCase(), ctx.url, {
            //   query: ctx.query,
            //   body: ctx.body,
            //   headers: ctx.headers,
            // })
            return ctx;
        });
        // Adjust origin param
        options.hooks.onBeforeRequest.push((ctx) => {
            const url = new URL(ctx.url);
            if (url.searchParams.has('origin')) {
                const origin = encodeURIComponent(url.searchParams.get('origin') || '').replace(/\./g, '%2E');
                delete ctx.query;
                url.searchParams.delete('origin');
                ctx.url = `${url}${url.search ? '&' : '?'}origin=${origin}`;
            }
            return ctx;
        });
        /**
         * response.data shortcut compatibility
         */
        options.hooks.onAfterResponse.push((ctx) => {
            Object.defineProperty(ctx, 'data', {
                get() {
                    try {
                        return JSON.parse(ctx.body);
                    }
                    catch (_) {
                        return ctx.body;
                    }
                },
            });
            return ctx;
        });
        // @ts-ignore FIXME: Type error during vite build, too bad!
        const { lyla } = (0, lyla_adapter_fetch_1.createLyla)({
            baseUrl: baseURL,
            ...options,
        });
        return lyla;
    }
    /** Syntactic Sugar */
    // request handler
    get request() {
        return tslib_1.__classPrivateFieldGet(this, _MediaWikiApi_requestHandler, "f").value;
    }
    // userOptions
    get defaultOptions() {
        return tslib_1.__classPrivateFieldGet(this, _MediaWikiApi_defaultOptions, "f").value;
    }
    set defaultOptions(options) {
        tslib_1.__classPrivateFieldGet(this, _MediaWikiApi_defaultOptions, "f").value = options;
    }
    // defaultParams
    get defaultParams() {
        return tslib_1.__classPrivateFieldGet(this, _MediaWikiApi_defaultParams, "f").value;
    }
    set defaultParams(params) {
        tslib_1.__classPrivateFieldGet(this, _MediaWikiApi_defaultParams, "f").value = params;
    }
    /** Base methods encapsulation */
    get(query, options) {
        return this.request.get(this.baseURL.value, {
            query: query,
            ...options,
        });
    }
    post(data, options) {
        return this.request.post(this.baseURL.value, {
            json: data,
            ...options,
        });
    }
    async login(lgname, lgpassword, params) {
        var _a, _b, _c, _d;
        this.defaultOptions.withCredentials = true;
        const { data } = await this.postWithToken('login', {
            action: 'login',
            lgname,
            lgpassword,
            ...params,
        }, { tokenName: 'lgtoken' });
        if (((_a = data === null || data === void 0 ? void 0 : data.login) === null || _a === void 0 ? void 0 : _a.result) !== 'Success') {
            throw new Error(((_c = (_b = data === null || data === void 0 ? void 0 : data.login) === null || _b === void 0 ? void 0 : _b.reason) === null || _c === void 0 ? void 0 : _c.text) || ((_d = data === null || data === void 0 ? void 0 : data.login) === null || _d === void 0 ? void 0 : _d.result) || 'Login failed');
        }
        return data.login;
    }
    async getUserInfo() {
        var _a;
        const { data } = await this.get({
            action: 'query',
            meta: 'userinfo',
            uiprop: ['groups', 'rights', 'blockinfo'],
        });
        return (_a = data === null || data === void 0 ? void 0 : data.query) === null || _a === void 0 ? void 0 : _a.userinfo;
    }
    /** Token Handler */
    async getTokens(type = ['csrf']) {
        this.defaultOptions.withCredentials = true;
        const { data } = await this.get({
            action: 'query',
            meta: 'tokens',
            type,
        });
        tslib_1.__classPrivateFieldSet(this, _MediaWikiApi_tokens, { ...tslib_1.__classPrivateFieldGet(this, _MediaWikiApi_tokens, "f"), ...data.query.tokens }, "f");
        return tslib_1.__classPrivateFieldGet(this, _MediaWikiApi_tokens, "f");
    }
    async token(type = 'csrf', noCache = false) {
        if (!tslib_1.__classPrivateFieldGet(this, _MediaWikiApi_tokens, "f")[`${type}token`] || noCache) {
            delete tslib_1.__classPrivateFieldGet(this, _MediaWikiApi_tokens, "f")[`${type}token`];
            await this.getTokens([type]);
        }
        return tslib_1.__classPrivateFieldGet(this, _MediaWikiApi_tokens, "f")[`${type}token`];
    }
    async postWithToken(tokenType, body, options) {
        const { tokenName = 'token', retry = 3, noCache = false } = options || {};
        if (retry < 1) {
            return Promise.reject({
                error: {
                    code: 'WIKI_SAIKOU_TOKEN_RETRY_LIMIT_EXCEEDED',
                    info: 'The limit of the number of times to automatically re-acquire the token has been exceeded',
                },
            });
        }
        const token = await this.token(tokenType, noCache);
        return this.post({
            [tokenName]: token,
            ...body,
        }).catch(({ data }) => {
            var _a, _b, _c;
            if ([(_a = data === null || data === void 0 ? void 0 : data.errors) === null || _a === void 0 ? void 0 : _a[0].code, (_b = data === null || data === void 0 ? void 0 : data.error) === null || _b === void 0 ? void 0 : _b.code].includes('badtoken') ||
                ['NeedToken', 'WrongToken'].includes((_c = data === null || data === void 0 ? void 0 : data.login) === null || _c === void 0 ? void 0 : _c.result)) {
                return this.postWithToken(tokenType, data, {
                    tokenName,
                    retry: retry - 1,
                    noCache: true,
                });
            }
            return Promise.reject(data);
        });
    }
    postWithEditToken(body) {
        return this.postWithToken('csrf', body);
    }
    async getMessages(ammessages, amlang = 'zh', options) {
        const { data } = await this.get({
            action: 'query',
            meta: 'allmessages',
            ammessages,
            amlang,
            ...options,
        });
        const result = {};
        data.query.allmessages.forEach(function (obj) {
            if (!obj.missing) {
                result[obj.name] = obj.content;
            }
        });
        return result;
    }
    async parseWikitext(wikitext, title, extraBody, options) {
        const { data } = await this.post({
            action: 'parse',
            title,
            text: wikitext,
            ...extraBody,
        }, options);
        return data.parse.text;
    }
}
exports.MediaWikiApi = MediaWikiApi;
exports.MwApi = MediaWikiApi;
_MediaWikiApi_requestHandler = new WeakMap(), _MediaWikiApi_defaultOptions = new WeakMap(), _MediaWikiApi_defaultParams = new WeakMap(), _MediaWikiApi_tokens = new WeakMap();
class MediaWikiForeignApi extends MediaWikiApi {
    constructor(baseURL, options) {
        super(baseURL, {
            withCredentials: true,
            ...options,
        });
        this.defaultParams.origin = location.origin;
    }
}
exports.MediaWikiForeignApi = MediaWikiForeignApi;
exports.ForeignApi = MediaWikiForeignApi;
// Aliases
exports.default = MediaWikiApi;
//# sourceMappingURL=index.js.map