"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.adapter = void 0;
function transformHeaders(headers) {
    if (!headers)
        return {};
    const headerMap = {};
    headers.forEach((value, key) => {
        headerMap[key] = value;
    });
    return headerMap;
}
const adapter = ({ url, method, headers, body, responseType, withCredentials, onDownloadProgress, onUploadProgress, onResponse, onNetworkError, }) => {
    const abortController = new AbortController();
    const request = fetch(url, {
        method,
        headers,
        body,
        credentials: withCredentials ? 'include' : 'same-origin',
        signal: abortController.signal,
    });
    request.then(async (response) => {
        let body;
        if (responseType === 'blob') {
            try {
                body = await response.clone().blob();
            }
            catch (error) { }
        }
        else if (responseType === 'arraybuffer') {
            try {
                body = await response.clone().arrayBuffer();
            }
            catch (error) { }
        }
        if (!body) {
            body = await response.clone().text();
        }
        onResponse({
            status: response.status,
            headers: transformHeaders(response.headers),
            body,
        }, response);
    });
    request.catch((error) => {
        onNetworkError(error);
    });
    return {
        abort() {
            abortController.abort();
        },
    };
};
exports.adapter = adapter;
//# sourceMappingURL=adapter.js.map